;~~~~~~~~~~~~~~~ L I C E N S E ~~~~~~~~~~~~~~~
;~ Copyright (c) 2015 chesstiger
;~
;~ This program is free software: you can redistribute it and/or modify
;~ it under the terms of the GNU Lesser General Public License as
;~ published by the Free Software Foundation, either version 3 of the
;~ License, or (at your option) any later version.
;~
;~ This program is distributed in the hope that it will be useful,
;~ but WITHOUT ANY WARRANTY; without even the implied warranty of
;~ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;~ GNU General Public License for more details.
;~
;~ You should have received a copy of the GNU Lesser General Public License
;~ along with this program.  If not, see <http://www.gnu.org/licenses/>.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include-once
#include <WindowsConstants.au3>
#include <GDIPlus.au3>
#include <WinAPI.au3>
#include <Array.au3>


Global $__g_oRFs = ObjCreate("Scripting.Dictionary")
Global $__g_hDllCB = DllCallbackRegister(__RedFrame_WM_PAINT, "ptr" , "hwnd;uint;long;long;")
OnAutoItExitRegister(__RedFrame_DisposeAll)
Global Enum $__RF_OLDWNDPROC, $__RF_GFX, $__RF_CTRLS
_GDIPlus_Startup()

; #FUNCTION# ====================================================================================================================
; Name ..........: _RedFrame_Set
; Description ...: Function to draw a red frame arround a controls
; Syntax ........: _RedFrame_Set($vCtrl, $iSpace[, $iWidth = 3])
; Parameters ....: $vCtrl               - control id or control handle
;                  $iSpace              - space between control and frame (px)
;                  $iWidth              - width of frame (px)
; Return values .: None
; Author ........: chesstiger (autoit.de)
; Modified ......:
; Remarks .......:
; Related .......: _RedFrame_Unset
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _RedFrame_Set($vCtrl, $iSpace, $iWidth = 3)
	If IsHWnd($vCtrl) Then
		$hCtrl = $vCtrl
	Else
		$hCtrl = GUICtrlGetHandle($vCtrl)
	EndIf
	$hParent = _WinAPI_GetParent($hCtrl)
	$hPen = _GDIPlus_PenCreate(0xFFFF0000, $iWidth)
	Local $aCtrl[3] = [$hCtrl, $iSpace, $hPen]

	If $__g_oRFs.Exists(String($hParent)) Then
		$aRF = $__g_oRFs(String($hParent))
		$aCtrls = $aRF[$__RF_CTRLS]
		ReDim $aCtrls[UBound($aCtrls) + 1]
		$aCtrls[UBound($aCtrls) - 1] = $aCtrl
		$aRF[$__RF_CTRLS] = $aCtrls
		$__g_oRFs(String($hParent)) = $aRF
	Else
		$hGfx = _GDIPlus_GraphicsCreateFromHWND($hParent)
		$pNewWndProc = DllCallbackGetPtr($__g_hDllCB)
		$pOldWndProc = _WinAPI_SetWindowLong($hParent, $GWL_WNDPROC, $pNewWndProc)

		Local $aCtrls[1] = [$aCtrl]
		Local $aRF[] = [$pOldWndProc, $hGfx, $aCtrls]
		$__g_oRFs(String($hParent)) = $aRF
	EndIf

	_WinAPI_RedrawWindow($hParent)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _RedFrame_Unset
; Description ...: Function to remove a control's red frame
; Syntax ........: _RedFrame_Unset($vCtrl)
; Parameters ....: $vCtrl               - control id or control handle
; Return values .: None
; Author ........: chesstiger (autoit.de)
; Modified ......:
; Remarks .......:
; Related .......: _RedFrame_Set
; Link ..........:
; Example .......: No
; ===============================================================================================================================
Func _RedFrame_Unset($vCtrl)
	If IsHWnd($vCtrl) Then
		$hCtrl = $vCtrl
	Else
		$hCtrl = GUICtrlGetHandle($vCtrl)
	EndIf
	$hParent = _WinAPI_GetParent($hCtrl)
	$aRF = $__g_oRFs(String($hParent))
	$aCtrls = $aRF[$__RF_CTRLS]
	$iEntry = -1 ;none/dummy
	For $i = 0 To UBound($aCtrls) - 1
		$aCtrl = $aCtrls[$i]
		If $hCtrl = $aCtrl[0] Then $iEntry = $i
	Next
	If $iEntry <> -1 Then _ArrayDelete($aCtrls, $iEntry)
	$aRF[$__RF_CTRLS] = $aCtrls
	$__g_oRFs(String($hParent)) = $aRF

	_WinAPI_RedrawWindow($hParent)
EndFunc

Func __RedFrame_WM_PAINT($hWnd, $iMsg, $iWParam, $iLParam)
	$aRF = $__g_oRFs(String($hWnd))

	If $iMsg = $WM_PAINT Then
		$tPaint = Null
		_WinAPI_BeginPaint($hWnd, $tPaint)
		$aCtrls = $aRF[$__RF_CTRLS]
		For $aCtrl In $aCtrls
			$hCtrl = $aCtrl[0]
			$iSpace = $aCtrl[1]
			$hPen = $aCtrl[2]
			$aPos = ControlGetPos($hWnd, "", _WinAPI_GetDlgCtrlID($hCtrl))
			_GDIPlus_GraphicsDrawRect($aRF[$__RF_GFX], $aPos[0] - $iSpace, $aPos[1] - $iSpace, $aPos[2] + $iSpace * 2, $aPos[3] + $iSpace * 2, $hPen)
		Next
		_WinAPI_EndPaint($hWnd, $tPaint)
		Return 0
	EndIf

	Return _WinAPI_CallWindowProc($aRF[$__RF_OLDWNDPROC], $hWnd, $iMsg, $iWParam, $iLParam)
EndFunc

Func __RedFrame_DisposeAll()
	For $sWnd In $__g_oRFs.Keys
		$hWnd = HWnd($sWnd)
		$pOldWndProc = ($__g_oRFs($sWnd))[$__RF_OLDWNDPROC]
		_WinAPI_SetWindowLong($hWnd, $GWL_WNDPROC, $pOldWndProc)
		_GDIPlus_GraphicsDispose(($__g_oRFs($sWnd))[$__RF_GFX])
		$aCtrls = ($__g_oRFs($sWnd))[$__RF_CTRLS]
		For $aCtrl In $aCtrls
			_GDIPlus_PenDispose($aCtrl[2])
		Next
	Next

	DllCallbackFree($__g_hDllCB)
	_GDIPlus_Shutdown()
EndFunc